#!/usr/bin/python
# -*- coding: utf-8 *-*

from __future__ import unicode_literals

__author__ = "Fco.Javier Tamarit <pymetrick@gmail.com>"
__copyright__ = "Copyright (C) 2023 PYMETRICK.ORG"
__license__ = "AGPLv3"
__version__ = '0.80.2'




